package org.apache.bcel.classfile;

import java.util.stream.Stream;
import java.util.Iterator;
import java.io.DataOutputStream;
import java.util.Arrays;
import java.io.IOException;
import java.io.DataInput;

public final class InnerClasses extends Attribute implements Iterable<InnerClass>
{
    private static final InnerClass[] EMPTY_INNER_CLASSE_ARRAY;
    private InnerClass[] innerClasses;
    
    public InnerClasses(final InnerClasses c) {
        this(c.getNameIndex(), c.getLength(), c.getInnerClasses(), c.getConstantPool());
    }
    
    InnerClasses(final int nameIndex, final int length, final DataInput input, final ConstantPool constantPool) throws IOException {
        this(nameIndex, length, (InnerClass[])null, constantPool);
        final int classCount = input.readUnsignedShort();
        this.innerClasses = new InnerClass[classCount];
        for (int i = 0; i < classCount; ++i) {
            this.innerClasses[i] = new InnerClass(input);
        }
    }
    
    public InnerClasses(final int nameIndex, final int length, final InnerClass[] innerClasses, final ConstantPool constantPool) {
        super((byte)6, nameIndex, length, constantPool);
        this.innerClasses = ((innerClasses != null) ? innerClasses : InnerClasses.EMPTY_INNER_CLASSE_ARRAY);
    }
    
    @Override
    public void accept(final Visitor v) {
        v.visitInnerClasses(this);
    }
    
    @Override
    public Attribute copy(final ConstantPool constantPool) {
        final InnerClasses c = (InnerClasses)this.clone();
        Arrays.setAll(c.innerClasses = new InnerClass[this.innerClasses.length], i -> this.innerClasses[i].copy());
        c.setConstantPool(constantPool);
        return c;
    }
    
    @Override
    public void dump(final DataOutputStream file) throws IOException {
        super.dump(file);
        file.writeShort(this.innerClasses.length);
        for (final InnerClass innerClass : this.innerClasses) {
            innerClass.dump(file);
        }
    }
    
    public InnerClass[] getInnerClasses() {
        return this.innerClasses;
    }
    
    @Override
    public Iterator<InnerClass> iterator() {
        return Stream.of(this.innerClasses).iterator();
    }
    
    public void setInnerClasses(final InnerClass[] innerClasses) {
        this.innerClasses = ((innerClasses != null) ? innerClasses : InnerClasses.EMPTY_INNER_CLASSE_ARRAY);
    }
    
    @Override
    public String toString() {
        final StringBuilder buf = new StringBuilder();
        buf.append("InnerClasses(");
        buf.append(this.innerClasses.length);
        buf.append("):\n");
        for (final InnerClass innerClass : this.innerClasses) {
            buf.append(innerClass.toString(super.getConstantPool())).append("\n");
        }
        return buf.substring();
    }
    
    static {
        EMPTY_INNER_CLASSE_ARRAY = new InnerClass[0];
    }
}
